@@ -64,6 +64,11 @@ CONSTANT_DTYPES_TO_NUMPY_TYPES = {
     'complex128': np.complex128,
 }
 
+ONNX_STANDARD_DOMAINS = [
+    'ai.onnx',
+    'ai.onnx.ml',
+    '',
+]
 
 def __search_op_constant_from_input_constant_name(
     graph: onnx_graphsurgeon.Graph,
@@ -140,6 +145,7 @@ def modify(
     attributes: Optional[dict] = None,
     delete_attributes: Optional[List[str]] = None,
     input_constants: Optional[dict] = None,
+    output_shapes: Optional[List] = None,
     non_verbose: Optional[bool] = False,
 ) -> onnx.ModelProto:
 
@@ -202,6 +208,20 @@ def modify(
         Default: None\n\
         https://github.com/onnx/onnx/blob/main/docs/Operators.md
 
+    output_shapes: Optional[List[int]]
+        Specifies the name of the output_shapes to be changed.\n\
+        output_shapes can be specified multiple times.\n\
+        output_shapes = [\n\
+            ['output_name1', shape1],\n\
+            ['output_name2', shape2],\n\
+                    :\n\
+        ]\n\
+        e.g.\n\
+        output_shapes = [\n\
+            ['aaa', [1]],\n\
+            ['bbb', [1,3,224,224]],\n\
+        ]
+
     non_verbose: Optional[bool]
         Do not show all information logs. Only error logs are displayed.\n\
         Default: False
@@ -223,8 +243,27 @@ def modify(
     # onnx_graph If specified, onnx_graph is processed first
     if not onnx_graph:
         onnx_graph = onnx.load(input_onnx_file_path)
+
+    # Acquisition of Node with custom domain
+    custom_domain_check_onnx_nodes = []
+    custom_domain_check_onnx_nodes = \
+        custom_domain_check_onnx_nodes + \
+            [
+                node for node in onnx_graph.graph.node \
+                    if node.domain not in ONNX_STANDARD_DOMAINS
+            ]
+
     graph = gs.import_onnx(onnx_graph)
 
+    # Check if Graph contains a custom domain (custom module)
+    contains_custom_domain = len(
+        [
+            domain \
+                for domain in graph.import_domains \
+                    if domain.domain not in ONNX_STANDARD_DOMAINS
+        ]
+    ) > 0
+
     # Search for OPs matching op_name
     node_subject_to_change = None
     if op_name:
@@ -260,6 +299,16 @@ def modify(
             node_subject_to_change.attrs = \
                 {attr_key: attr_value for attr_key, attr_value in node_subject_to_change.attrs.items() if attr_key not in delete_attributes}
 
+    # Updating Ouput shapes
+    if node_subject_to_change:
+        # Update
+        if output_shapes:
+            for outupt_name, output_shape in output_shapes:
+                for output in node_subject_to_change.outputs:
+                    if output.name == outupt_name:
+                        output.shape = output_shape
+
+
     # Updating Constants
     """
     graph_node
@@ -355,6 +404,14 @@ def modify(
             tracetxt = traceback.format_exc().splitlines()[-1]
             print(f'{Color.YELLOW}WARNING:{Color.RESET} {tracetxt}')
 
+    ## Restore a node's custom domain
+    if contains_custom_domain:
+        new_model_nodes = new_model.graph.node
+        for new_model_node in new_model_nodes:
+            for custom_domain_check_onnx_node in custom_domain_check_onnx_nodes:
+                if new_model_node.name == custom_domain_check_onnx_node.name:
+                    new_model_node.domain = custom_domain_check_onnx_node.domain
+
     # Save
     if output_onnx_file_path:
         onnx.save(new_model, f'{output_onnx_file_path}')
@@ -368,18 +425,21 @@ def modify(
 def main():
     parser = ArgumentParser()
     parser.add_argument(
+        '-if',
         '--input_onnx_file_path',
         type=str,
         required=True,
         help='Input onnx file path.'
     )
     parser.add_argument(
+        '-of',
         '--output_onnx_file_path',
         type=str,
         required=True,
         help='Output onnx file path.'
     )
     parser.add_argument(
+        '-on',
         '--op_name',
         type=str,
         help=\
@@ -388,6 +448,7 @@ def main():
             'e.g. --op_name aaa'
     )
     parser.add_argument(
+        '-a',
         '--attributes',
         nargs=3,
         action='append',
@@ -404,6 +465,7 @@ def main():
             '--attributes transB int64 0'
     )
     parser.add_argument(
+        '-da',
         '--delete_attributes',
         nargs='+',
         help=\
@@ -416,6 +478,7 @@ def main():
             '--delete_attributes alpha beta'
     )
     parser.add_argument(
+        '-ic',
         '--input_constants',
         type=str,
         nargs=3,
@@ -431,6 +494,22 @@ def main():
             '--input_constants constant_name2 float32 [[1.0,2.0,3.0],[4.0,5.0,6.0]]'
     )
     parser.add_argument(
+        '-os',
+        '--output_shapes',
+        type=str,
+        nargs=2,
+        action='append',
+        help=\
+            'Specifies the name of the output_shapes to be changed. \n'+
+            'output_shapes can be specified multiple times. \n'+
+            '--output_shapes output_name1 shape1 \n'+
+            '--output_shapes output_name2 shape2 \n\n'+
+            'e.g.\n'+
+            '--output_shapes output_name1 [1] \n'+
+            '--output_shapes output_name2 [1,3,224,224]'
+    )
+    parser.add_argument(
+        '-n',
         '--non_verbose',
         action='store_true',
         help='Do not show all information logs. Only error logs are displayed.'
@@ -440,6 +519,7 @@ def main():
     input_onnx_file_path = args.input_onnx_file_path
     output_onnx_file_path = args.output_onnx_file_path
     input_constants = args.input_constants
+    output_shapes = args.output_shapes
     op_name = args.op_name
     attributes = args.attributes
     delete_attributes = args.delete_attributes
@@ -468,6 +548,14 @@ def main():
         )
         sys.exit(1)
 
+    # op_name and output_shapes must always be specified at the same time.
+    if (not op_name and output_shapes):
+        print(
+            f'{Color.RED}ERROR:{Color.RESET} '+
+            f'op_name and output_shapes must always be specified at the same time.'
+        )
+        sys.exit(1)
+
     # attributes
     """
     attributes_tmp = {"name": value}
@@ -481,10 +569,39 @@ def main():
             attr_type = attribute[1]
             if attr_type == 'string':
                 attr_type = 'str'
-            if attr_type != 'str':
-                attr_value = ast.literal_eval(attribute[2])
-            else:
+            if attr_type == 'str':
                 attr_value = attribute[2]
+            else:
+                if ('-inf' in attribute[2].lower()) or ('-infinity' in attribute[2].lower()):
+                    lower_attr = attribute[2].lower()
+                    inf_count = lower_attr.count('-inf')
+                    infinity_count = lower_attr.count('-infinity')
+                    if (inf_count + infinity_count) > 1:
+                        print(
+                            f'{Color.RED}ERROR:{Color.RESET} '+
+                            f'Values containing "inf" or "-inf" can only be 1D tensors. \n'+
+                            f'e.g. [inf] or [-inf]'
+                        )
+                        sys.exit(1)
+                    else:
+                        attr_value = [-np.inf]
+
+                elif ('inf' in attribute[2].lower()) or ('infinity' in attribute[2].lower()):
+                    lower_attr = attribute[2].lower()
+                    inf_count = lower_attr.count('inf')
+                    infinity_count = lower_attr.count('infinity')
+                    if (inf_count + infinity_count) > 1:
+                        print(
+                            f'{Color.RED}ERROR:{Color.RESET} '+
+                            f'Values containing "inf" or "-inf" can only be 1D tensors. \n'+
+                            f'e.g. [inf] or [-inf]'
+                        )
+                        sys.exit(1)
+                    else:
+                        attr_value = [np.inf]
+
+                else:
+                    attr_value = ast.literal_eval(attribute[2])
 
             # dtype check
             if attr_type not in ATTRIBUTE_DTYPES_TO_NUMPY_TYPES:
@@ -537,6 +654,19 @@ def main():
 
             input_constants_tmp[constant_name] = constant_value
 
+    # output_shapes
+    # output_shapes = [output_name, shape]
+    output_shapes_tmp = None
+    if output_shapes:
+        output_shapes_tmp = []
+        for output_name, output_shape in output_shapes:
+            output_shapes_tmp.append(
+                [
+                    output_name,
+                    ast.literal_eval(output_shape),
+                ]
+            )
+
     # Model modify
     modified_graph = modify(
         input_onnx_file_path=None,
@@ -546,6 +676,7 @@ def main():
         attributes=attributes_tmp,
         delete_attributes=delete_attributes,
         input_constants=input_constants_tmp,
+        output_shapes=output_shapes_tmp,
         non_verbose=non_verbose
     )
 
